# $Id: queuemodel.py,v 1.16 2010/11/22 17:44:57 irees Exp $
import os
import operator
import collections
import time
import datetime

from PyQt4 import QtCore, QtGui, Qt


class QueueFile(dict):
	pass
		


# These two classes adapted from Qt Examples, for obvious reasons..
class DictItem(object):	
	def __init__(self, data, parent=None, cols=None):
		self.parentItem = parent
		self.data = data
		self.childItems = []
		self.cols = cols or []

	def get(self, key, default=None):
		return self.data.get(key, default)

	def appendChild(self, item):
		self.childItems.append(item)

	def child(self, row):
		try: return self.childItems[row]
		except: return None

	def childCount(self):
		return len(self.childItems)

	def childNumber(self):
		if self.parentItem != None:
			return self.parentItem.childItems.index(self)
		return 0

	def columnCount(self):
		return len(self.data)

	def data(self, column):
		# cols = ["_recname", "tem_magnification_set", "ctf_defocus_set", 'tem_dose_rate', 'time_exposure_tem', "assess_image_quality", "_status"]
		return self.data.get(self.cols[column])

	def parent(self):
		return self.parentItem		
		
		

class QueueModel(QtCore.QAbstractItemModel):
	itemclass = DictItem
	
	def __init__(self, headers=None, headernames=None, queue=None, parent=None):
		super(QueueModel, self).__init__(parent)
		self.queue = queue
				
		default_headers = ["_recname", "_status"]
		default_headernames = ["Name", "Status"]
		self.headers = headers or default_headers
		self.headernames = headernames or default_headernames

		self.rootItem = self.itemclass(QueueFile({}), parent=None, cols=self.headers)
		
		self.recids = {}
		self.grids = {}
		self.mdata = {}



	def newgrid(self, recid):
		# Begin a new upload target...
		self.recids["grid_imaging"] = recid
		item = {"recid":recid, "_recname":"Grid: %s"%recid, "_status":"(view)"}
		# ian: fix this..
		self.beginInsertRows(QtCore.QModelIndex(), self.rowCount(), self.rowCount())			
		treeitem = self.itemclass(item, parent=self.rootItem, cols=self.headers)
		self.rootItem.appendChild(treeitem)
		self.endInsertRows()
		
		# Save the current recid, so new items in the queue will be sent to it
		self.grids[recid] = treeitem

		
		
	# @QtCore.pyqtSlot(unicode)
	def newfile(self, filename, item):
		"""Add a file to the model"""

		filename = unicode(filename)
		parent = self.recids.get("grid_imaging")		
		item.update({"_filename":filename, "_parent":parent})

		self.beginInsertRows(QtCore.QModelIndex(), self.rowCount(), self.rowCount())

		d = self.itemclass(item, parent=self.grids[parent], cols=self.headers)
		self.grids[parent].appendChild(d)
		self.endInsertRows()
				
		self.mdata[filename] = d
		if item.get('_auto') == True:
			self.queue.put(item)
	
	
	def file_status(self, filename, d):
		"""Update the status dict for a file"""
		self._updatestatus(filename, d)


	def file_success(self, filename, ret):
		"""A file has been successfully uploaded into the db"""
		try:
			ret['_status'] = "%s"%ret.get("recid")
			self._updatestatus(filename, ret)
		except Exception, inst:
			pass


	def file_failure(self, filename, e):
		"""There was an error uploading file"""
		self._updatestatus(filename, {"_error": e, "_status": "Retry: %s"%e})


	def _updatestatus(self, filename, d):
		data = self.mdata.get(unicode(filename))
		if not data:
			for k,v in self.mdata.items():
				if str(v.get('recid')) == filename:
					data = v
							
		if not data:
			return
				
		try:
			data.data.update(d)
		except KeyError:
			return

		parentItem = data.parent()
		i = self.createIndex(parentItem.childNumber(), 0, parentItem)
		j = self.createIndex(parentItem.childNumber(), 4, parentItem)
		self.dataChanged.emit(i,j)


	######################
	

	def data(self, index, role):
		if not index.isValid():
			return None

		if role != QtCore.Qt.DisplayRole and role != QtCore.Qt.EditRole:
			return None

		item = self.getItem(index)
		col = self.headers[index.column()]		
		return item.data.get(col)


	def getmdata(self, index):
		item = self.getItem(index)
		return item.data


	def flags(self, index):
		if not index.isValid():
			return 0
		return QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable


	def getItem(self, index):
		if index.isValid():
			item = index.internalPointer()
			if item:
				return item
		return self.rootItem


	def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
		if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
			return self.headernames[section]
		return None

	def index(self, row, column, parent=QtCore.QModelIndex()):
		if parent.isValid() and parent.column() != 0:
			return QtCore.QModelIndex()

		parentItem = self.getItem(parent)
		childItem = parentItem.child(row)

		if childItem:
			return self.createIndex(row, column, childItem)
		else:
			return QtCore.QModelIndex()


	def parent(self, index):
		if not index.isValid():
			return QtCore.QModelIndex()

		childItem = self.getItem(index)
		parentItem = childItem.parent()

		if parentItem == self.rootItem:
			return QtCore.QModelIndex()

		return self.createIndex(parentItem.childNumber(), 0, parentItem)


	def rowCount(self, parent=QtCore.QModelIndex()):
		parentItem = self.getItem(parent)
		return parentItem.childCount()


	def columnCount(self, parent=QtCore.QModelIndex()):
		return len(self.headers)








__version__ = "$Revision: 1.16 $".split(":")[1][:-1].strip()
